import os as _os
on_rtd = _os.environ.get('READTHEDOCS', None) == 'True'
if not on_rtd:
    import numpy as _np
    import slactrac as _sltr


class Plasma(object):
    """
    A class for relating plasma density to plasma frequency :math:`\\omega_p` and ion focusing force.

    Input either:

    * Plasma density *n_p* in SI units
    * Plasma density *n_p_cgs* in CGS units
    """
    def __init__(self, n_p=None, n_p_cgs=None):
        if n_p is None and n_p_cgs is None:
            raise ValueError('Keywords n_p and n_p_cgs cannot both be None')
        elif n_p is not None and n_p_cgs is not None:
            raise ValueError('Keywords n_p and n_p_cgs cannot both be specified')
        elif n_p is not None:
            self.n_p = n_p
        elif n_p_cgs is not None:
            self.n_p_cgs = n_p_cgs

    @property
    def n_p(self):
        """
        Plasma density in SI units
        """
        return self._n_p

    @n_p.setter
    def n_p(self, n_p):
        self._n_p = n_p

    @property
    def w_p(self):
        """
        Plasma frequency :math:`\\omega_p` for given plasma density
        """
        return _np.sqrt(self.n_p * _np.power(_sltr.e, 2) / (_sltr.m_e * _sltr.epsilon_0))

    def k_ion(self, E):
        """
        Geometric focusing force due to ion column for given plasma density as a function of *E*
        """
        return self.n_p * _np.power(_sltr.e, 2) / (2*_sltr.GeV2joule(E) * _sltr.epsilon_0)

    @property
    def n_p_cgs(self):
        """
        Plasma density in CGS units
        """
        return self.n_p / 1e6

    @n_p_cgs.setter
    def n_p_cgs(self, n_p_cgs):
        self.n_p = n_p_cgs * 1e6



"""
The :mod:`numpy <scisalt.numpy>` module contains a few convenience functions mostly designed to make evaluating functions easier for plotting.
"""
__all__ = ['frexp10', 'linspaceborders', 'linspacestep']
from .frexp10 import *
from .linspaceborders import *
from .linspacestep import *



# Author: Joel Frederico
"""
This project has slowly grown from frequently-used code snippets. The main objective is to create a collection of interconnected methods frequently needed to visualize and analyze data using `NumPy <http://www.numpy.org/>`_, `SciPy <http://www.scipy.org/>`_, `Matplotlib <http://matplotlib.org/>`_, and `PyQt4 <http://www.riverbankcomputing.com/software/pyqt/download>`_.
"""
__version__ = '1.4.5'
from . import accelphys
from . import facettools
from . import logging
from . import matplotlib
from . import numpy
from . import scipy
from . import utils

# class Error(Exception):
#     """Base class for exceptions in this module."""
#     pass
#
#
# class ArgumentError(Error):
#     """Exception raised for errors in function arguments.
#
#     Attributes:
#         arg -- input argument in which the error occurred
#         msg -- explanation of the error
#     """
#
#     def __init__(self, arg, msg):
#         self.arg = arg
#         self.msg = msg



import sys as _sys


class progressbar(object):
    """
    .. versionadded:: 1.3

    Creates a progress bar with a number of steps *total* and a length of *length*.
    """
    def __init__(self, total, length=20):
        print('')
        self._step   = 0
        self._total  = total
        self._length = length

    @property
    def step(self):
        """
        The current step.
        """
        return self._step

    @step.setter
    def step(self, step):
        if step > self._total:
            step = self._total
        self._step = step
        bartext = '#'*round(step/self._total * self._length) + ' '*round((self._total-step)/self._total * self._length)
        text = '\r\033[1AOn step {} of {}:\n[ {} ]'.format(self._step, self._total, bartext)
        _sys.stdout.write(text)
        _sys.stdout.flush()
        # print(text)

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        text = '\r\033[K\033[1A\033[K'
        _sys.stdout.write(text)
        _sys.stdout.flush()



import os as _os
on_rtd = _os.environ.get('READTHEDOCS', None) == 'True'
if not on_rtd:
    import numpy as _np
    import slactrac as _sltr
from .plasma import *


class Match(object):
    """
    Given a *plasma* of type :class:`Plasma <scisalt.PWFA.Plasma>` and a beam of energy *E* in GeV and normalized emittance *emit_n* in SI units, calculates match parameters
    """
    def __init__(self, plasma, E, emit_n):
        self.plasma = plasma
        self.E = E
        self.emit_n = emit_n

    @property
    def gamma(self):
        """
        Relativistic :math:`\\gamma` of beam
        """
        return _sltr.GeV2gamma(self.E)

    @gamma.setter
    def gamma(self, value):
        self.E = _sltr.gamma2GeV(value)

    @property
    def emit_n(self):
        """
        Emittance of beam
        """
        return self.emit * self.gamma

    @emit_n.setter
    def emit_n(self, value):
        self.emit = value / self.gamma

    @property
    def sigma(self):
        """
        Spot size of matched beam
        """
        return _np.power(2*_sltr.GeV2joule(self.E)*_sltr.epsilon_0 / (self.plasma.n_p * _np.power(_sltr.e, 2)) , 0.25) * _np.sqrt(self.emit)

    def beta(self, E):
        """
        :math:`\\beta` function of matched beam
        """
        return 1.0 / _np.sqrt(self.plasma.k_ion(E))
        # return 1.0 / _np.sqrt(2)
